#vider tout
rm(list=ls())

#fonction de calcul taux d'erreur
calculer_taux_erreur <- function(modele,donnees){
  #obtenir la proba d'tre positif
  pplus <- predict(modele,newdata=donnees,type="response")
  #obtenir la prdiction
  prediction <- as.factor(ifelse(pplus > 0.5,"positive","negative"))
  #calculer la matrice de confusion
  mc <- table(donnees.test$diabete,prediction)
  print(mc)
  #calculer le taux d'erreur
  err.rate <- (mc[2,1]+mc[1,2])/sum(mc)
  print(err.rate)
}

#charger le package "xlsx"
library(xlsx) #lecture directe des fichiers EXCEL

#charger les donnes apprentissage
donnees.app <- read.xlsx(file="diabete_reg_logistique.xlsx",sheetIndex="apprentissage")

#caractristiques
print(str(donnees.app))

#summary
print(summary(donnees.app))

#donnes test
donnees.test <- read.xlsx(file="diabete_reg_logistique.xlsx",sheetIndex="test")

#caractristiques
print(str(donnees.test))

#summary
print(summary(donnees.test))

#rgression logistique - diabete vs. les autres variables
modele <- glm(diabete ~ ., data = donnees.app, family = binomial)
print(modele)

#attributs de l'objet modele
print(attributes(modele))

#R McFadden
LLa <- modele$deviance/(-2)
LL0 <- modele$null.deviance/(-2)
R2MF <- 1.0 - LLa/LL0
print(R2MF)

#R2 Cox-Snell
La <- exp(LLa)
L0 <- exp(LL0)
R2CS <- 1.0 - (L0/La)^(2/nrow(donnees.app))
print(R2CS)

#R2 Nagelkerke
R2N <- R2CS/(1.0 - L0^(2/nrow(donnees.app)))
print(R2N)

#test de significativ globale

#chi-2
chi2 <- modele$null.deviance - modele$deviance
print(chi2)

#ddl
ddl <- modele$df.null - modele$df.residual
print(ddl)

#p-value
pvalue <- pchisq(chi2,ddl,lower.tail = FALSE)
print(pvalue)

#prdiction
proba <- predict(modele, newdata = donnees.test, type = "response")
print(proba)

#prdiction
pred <- factor(ifelse(proba > 0.5, "positive", "negative"))

#distribution des prdiction
print(table(pred))

#matrice de confusion
mc <- table(donnees.test$diabete,pred)
print(mc)

#taux d'erreur
err <- 1.0-sum(diag(mc))/sum(mc)
print(err)

#appel de la fonction pour le calcul du taux d'erreur
calculer_taux_erreur(modele,donnees.test)

#test de Wald pour valuation des variables
res <- summary(modele)
print(res)

#pour stepAIC
library(MASS)

#selection backward
modele.backward_AIC <- stepAIC(modele, data = donnees.app, direction="backward")
summary(modele.backward_AIC)
calculer_taux_erreur(modele.backward_AIC,donnees.test)

#selection backward - BIC
modele.backward_BIC <- stepAIC(modele, data = donnees.app, direction="backward", k = log(nrow(donnees.app)))
summary(modele.backward_BIC)
calculer_taux_erreur(modele.backward_BIC,donnees.test)

#proprits
print(attributes(modele.backward_BIC))

#champ ANOVA
print(modele.backward_BIC$anova)

#graphique
plot(0:(nrow(modele.backward_BIC$anova)-1),modele.backward_BIC$anova[,"AIC"],type="b",xlab="# de var. retires",ylab="BIC",main="Slection backward (BIC)")

#slection forward - BIC
modele.initial <- glm(diabete ~ 1, data = donnees.app, family = binomial)
modele.forward <- stepAIC(modele.initial, data = donnees.app, scope = list(lower="~1",upper=as.formula(modele)),direction="forward",k=log(nrow(donnees.app)))
summary(modele.forward)

#ordre de slection
print(modele.forward$anova)
